#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// In March 2007 TASC
    /// </summary>
    [Description("Fractal Dimension Index")]
    public class FDI : Indicator
    {
        #region Variables
        // Wizard generated variables
		private int indicatorPeriod = 30; // Default setting for Period
		
		
 		private double Diff = 0; 
		private double Length = 0; 
		private double PriceMax = 0; 
 		private double PriceMin = 0; 
    	private double PriorDiff = 0; 
    	private double Iteration = 0; 
    	private double FractalDI = 0;		
		
		
//		private double calculatedFDI;
//		private double HurstExponent;
//		private	double StdDeviationPeriod;
//		private	double RangeOfCulmativeDeviation;
//			
//		
//		private DataSeries LogarithmicReturn;
//		private DataSeries AverageReturn;
//		private DataSeries CulmativeDeviation;
//		private DataSeries LogarithmicReturnMinusAverageReturn;
		
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(new Pen(Color.Magenta, 2), PlotStyle.Line, "IndexAbove"));
            Add(new Plot(new Pen(Color.LimeGreen, 2), PlotStyle.Line, "IndexBelow"));
			Add(new Line(System.Drawing.Color.Red, 1.5, "TrendLvl"));
			
			CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= true;
			
			Plots[0].Min = 1.5; 
			Plots[1].Max = 1.5;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {		
			if(CurrentBar < indicatorPeriod)
				return;
   
			PriceMax = MAX( Close, indicatorPeriod )[0] ; 
			PriceMin = MIN( Close, indicatorPeriod )[0] ; 
			Length = 0 ; 
			PriorDiff = 0 ; 
			for (int idx = 1; idx  < indicatorPeriod; idx++)	
			{
				if (PriceMax - PriceMin > 0) 
				{
					Diff = (Close[idx] - PriceMin) / (PriceMax - PriceMin) ; 
					if (idx > 1) Length = (Length + Math.Sqrt( Math.Pow((Diff - PriorDiff) + (1 / Math.Pow(indicatorPeriod,2)), 2))) ;  
					PriorDiff = Diff  ;  
				}
			}
 
			if (Length > 0) FractalDI = 1 + ( Math.Log( Length )+ Math.Log( 2 ) ) / Math.Log( 2 * ( indicatorPeriod ) );
			else FractalDI = 0 ; 
				
			IndexAbove.Set(FractalDI);
			IndexBelow.Set(FractalDI);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries IndexAbove
        {
            get { return Values[0]; }
        }		
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries IndexBelow
        {
            get { return Values[1]; }
        }

        [Description("Period for getting Fractal Dimension")]
        [Category("Parameters")]
        public int IndicatorPeriod
        {
            get { return indicatorPeriod; }
			set { indicatorPeriod = Math.Max(1, value); }
        }
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private FDI[] cacheFDI = null;

        private static FDI checkFDI = new FDI();

        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        public FDI FDI(int indicatorPeriod)
        {
            return FDI(Input, indicatorPeriod);
        }

        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        public FDI FDI(Data.IDataSeries input, int indicatorPeriod)
        {
            checkFDI.IndicatorPeriod = indicatorPeriod;
            indicatorPeriod = checkFDI.IndicatorPeriod;

            if (cacheFDI != null)
                for (int idx = 0; idx < cacheFDI.Length; idx++)
                    if (cacheFDI[idx].IndicatorPeriod == indicatorPeriod && cacheFDI[idx].EqualsInput(input))
                        return cacheFDI[idx];

            FDI indicator = new FDI();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.IndicatorPeriod = indicatorPeriod;
            indicator.SetUp();

            FDI[] tmp = new FDI[cacheFDI == null ? 1 : cacheFDI.Length + 1];
            if (cacheFDI != null)
                cacheFDI.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheFDI = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FDI FDI(int indicatorPeriod)
        {
            return _indicator.FDI(Input, indicatorPeriod);
        }

        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        public Indicator.FDI FDI(Data.IDataSeries input, int indicatorPeriod)
        {
            return _indicator.FDI(input, indicatorPeriod);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FDI FDI(int indicatorPeriod)
        {
            return _indicator.FDI(Input, indicatorPeriod);
        }

        /// <summary>
        /// Fractal Dimension Index
        /// </summary>
        /// <returns></returns>
        public Indicator.FDI FDI(Data.IDataSeries input, int indicatorPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.FDI(input, indicatorPeriod);
        }

    }
}
#endregion
